import chalk from 'chalk'
import fs from 'node:fs'
import { fileURLToPath } from 'node:url'
import path from 'node:path'
import minimist from 'minimist'
import prompts from 'prompts'
import { blue, green, red, reset, yellow } from 'kolorist'
import { createProject } from './main.js'

const __filename = fileURLToPath(import.meta.url)
const __dirname = path.dirname(__filename)
let packageJSON = fs.readFileSync(path.join(__dirname, '../package.json'), 'utf-8')
packageJSON = JSON.parse(packageJSON)
const { version } = packageJSON

const defaultTargetDir = 'vue3-project'

const FRAMEWORKS = [
  {
    name: 'gx-design-pro',
    color: red
  },
  {
    name: 'gx-design-pro-thin',
    color: blue
  },
  {
    name: 'mobile-vant-cli',
    color: yellow
  },
  {
    name: 'mobile-vant-html',
    color: green
  }
]

const TEMPLATES = FRAMEWORKS.map(
  f => (f.variants && f.variants.map(v => v.name)) || [ f.name ]
).reduce((a, b) => a.concat(b), [])

function formatTargetDir(targetDir) {
  return targetDir ? targetDir.trim().replace(/\/+$/g, '') : ''
}

function parseArgumentsIntoOptions() {
  const argv = minimist(process.argv.slice(2), { string: [ '_' ] })
  const argTargetDir = formatTargetDir(argv._[0])
  const argTemplate = argv.template || argv.t
  return {
    v: argv.v,
    template: argTemplate,
    projectName: argTargetDir
  }
}

async function promptForMissingOptions(options) {
  const projectNamePrompts = []
  if (!options.projectName) {
    projectNamePrompts.push({
      type: options.projectName ? null : 'text',
      name: 'projectName',
      message: reset('Project name:'),
      initial: defaultTargetDir
    })
  }
  if (typeof options.template !== 'string' || !TEMPLATES.includes(options.template)) {
    projectNamePrompts.push({
      type: 'select',
      name: 'template',
      message:
        typeof options.template === 'string' && !TEMPLATES.includes(options.template)
          ? reset(
            `"${options.template}" isn't a valid template. Please choose from below: `
          )
          : reset('Select a Template:'),
      choices: FRAMEWORKS.map((framework) => {
        const frameworkColor = framework.color
        return {
          title: frameworkColor(framework.name),
          value: framework
        }
      })
    })
  }
  let answers = {
    template: {
      name: ''
    },
    projectName: ''
  }

  if (projectNamePrompts.length) {
    answers = await prompts(projectNamePrompts)
  }

  return {
    ...options,
    template: options.template || (answers.template ? answers.template.name : ''),
    projectName: options.projectName || answers.projectName || ''
  }
}

export async function cli() {
  let options = parseArgumentsIntoOptions()
  if (options.v) {
    console.log(chalk.blueBright(version))
    process.exit(1)
  }
  options = await promptForMissingOptions(options)
  if (options.template && options.projectName) {
    await createProject(options)
  } else {
    process.exit(1)
  }
}
