import fs from 'node:fs'
import path from 'node:path'
import { fileURLToPath } from 'node:url'
import { promisify } from 'node:util'
import chalk from 'chalk'
import { reset } from 'kolorist'
import Listr from 'listr'
import { createSpinner } from 'nanospinner'
import ncp from 'ncp'
import prompts from 'prompts'
import { clone } from './git.js'

const copy = promisify(ncp)

const gitRepositoryList = [
  {
    name: 'gx-design-pro',
    children: [
      {
        name: 'Gitee',
        url: 'https://gitee.com/gx12358/vue3-antd-admin.git'
      },
      {
        name: 'Github',
        url: 'https://github.com/gx12358/vue3-antd-admin.git'
      }
    ]
  },
  {
    name: 'gx-design-pro-thin',
    children: [
      {
        name: 'Gitee',
        url: 'https://github.com/gx12358/vue3-antd-admin-thin.git'
      },
      {
        name: 'Github',
        url: 'https://github.com/gx12358/vue3-antd-admin-thin.git'
      }
    ]
  }
]

async function copyTemplateFiles(templateDir, root) {
  const exclusions = [ 'node_modules' ]
  return copy(templateDir, root, {
    clobber: false,
    filter: (sourcePath) => {
      const relativePath = path.relative(templateDir, sourcePath)
      const folders = relativePath.split(path.sep)
      const folderToCheck = folders.length > 1 ? folders[1] : folders[0]
      return !exclusions.includes(folderToCheck)
    }
  })
}

function writeFile(path, content) {
  if (Object.keys(content).length > 0) {
    fs.writeFileSync(path, content)
  }
}

function editPackageJson(root, projectName) {
  let pkg = {}

  if (fs.existsSync(path.join(root, `package.json`))) {
    pkg = JSON.parse(
      fs.readFileSync(path.join(root, `package.json`), 'utf-8')
    )

    pkg.name = projectName

    const dirIsExists = fs.existsSync(path.join(root, '.npmrc'))

    if (!dirIsExists) {
      writeFile(path.join(root, '.npmrc'), 'shamefully-hoist=true')
    }

    writeFile(path.join(root, 'package.json'), JSON.stringify(pkg, null, 2))
  }
}

function doneLog(cwd, root) {
  if (root !== cwd) {
    console.log(chalk.blueBright(`cd ${path.relative(cwd, root)}`))
  }
  console.log(chalk.blueBright(`pnpm、yarn、npm install`))
  console.log(chalk.blueBright(`pnpm、yarn、npm run dev`))
  console.log()
}

function getProjectName(projectName) {
  return projectName === '.' ? path.basename(path.resolve()) : projectName
}

function download(url, root, projectName) {
  const spinner = createSpinner(`开始下载...${url}`).start()
  // 下载git代码
  clone(`${url}`, root, { clone: true }, async () => {
    await editPackageJson(root, projectName || getProjectName(projectName))
    spinner.success({
      text: '项目创建成功，请依次执行以下命令'
    })
    doneLog(process.cwd(), root)
  }, async (err) => {
    console.log(err)
    spinner.error({ text: '下载失败' })
  })
}

export async function createProject(options) {
  const cwd = process.cwd()

  const root = path.join(cwd, options.projectName)

  const downloadGit = gitRepositoryList.find(el => el.name === options.template)

  const templateDir = path.resolve(
    fileURLToPath(import.meta.url),
    '../..',
    `template-${options.template}`
  )

  const dirIsTemplate = fs.existsSync(templateDir)

  if (!dirIsTemplate) {
    if (!downloadGit) {
      console.log(chalk.redBright('模版目录不存在'))
      process.exit(1)
    }
  }

  // 目录是否已经存在
  const dirIsExists = fs.existsSync(root)

  if (dirIsExists) {
    console.log(chalk.redBright('目录已经存在'))
    process.exit(1)
  }

  if (downloadGit) {
    if (downloadGit.children && downloadGit.children.length > 0) {
      prompts([
        {
          type: 'select',
          name: 'url',
          message: reset('Select a Url:'),
          choices: downloadGit.children.map((item) => {
            return {
              title: item.name,
              value: item.url
            }
          })
        }
      ]).then((answers) => {
        if (answers.url) download(answers.url, root, options.projectName)
      })
    } else if (downloadGit.url) {
      download(downloadGit.url, root, options.projectName)
    }
  } else {
    const listr = [
      {
        title: '项目创建',
        task: () => copyTemplateFiles(templateDir, root)
      }
    ]
    if (options.template !== 'mobile-vant-html') {
      listr.push({
        title: '初始化项目',
        task: () => editPackageJson(root, options.projectName || getProjectName(options.projectName))
      })
    }
    const tasks = new Listr(
      listr,
      {
        exitOnError: false
      }
    )

    await tasks.run()

    if (options.template !== 'mobile-vant-html') {
      doneLog(cwd, root)
    } else {
      console.log(chalk.blueBright(`init success`))
    }
  }
  return true
}
