import type { SetLoadingParams } from '@gx-mobile/hooks/web/usePageLoading'
import type { NavBarProps } from 'vant'
import { classNames, getSlot, getSlotVNode, isFunction, VueNode, WithFalse } from '@gx-design-vue/pro-utils'
import { usePageLoading } from '@gx-mobile/hooks/web'
import { useScroll } from '@vueuse/core'
import { Icon, NavBar } from 'vant'
import { defineComponent, ref, type SlotsType, Teleport, toRefs, watch } from 'vue'
import ProSkeleton from './ProSkeleton'
import styles from './style.module.less'
import 'vant/es/nav-bar/style'

export type PageLoadingTpe = 'toast' | 'skeleton'

type ChangeLoadingParams<T> = Omit<SetLoadingParams<T>, 'type'> & {
  hiddenSlot?: boolean;
}

const PageContainer = defineComponent({
  name: 'GPageContainer',
  inheritAttrs: false,
  props: {
    loading: {
      type: Boolean as VuePropType<boolean>,
      default: false
    },
    nav: {
      type: [ Boolean, Function, Object, Array ] as VuePropType<WithFalse<VueNode>>,
      default: false
    },
    navProps: Object as VuePropType<NavBarProps>,
    backTop: {
      type: [ Boolean, Function, Object, Array ] as VuePropType<WithFalse<VueNode>>,
      default: false
    },
    wrapperClass: String as VuePropType<string>,
    backTopProps: {
      type: Object as VuePropType<{
        style: CSSProperties;
        count: number;
      }>,
      default: () => ({
        count: 300
      })
    },
    hiddenSlot: {
      type: Boolean as VuePropType<boolean>,
      default: true
    },
    loadingMsg: {
      type: String as VuePropType<string>,
      default: '加载中'
    },
    loadingType: {
      type: String as VuePropType<PageLoadingTpe>,
      default: 'toast'
    },
    bgColor: {
      type: String as VuePropType<string>,
      default: 'var(--van-gray-1)'
    },
    onNavClickLeft: Function as VuePropType<() => void>,
    'onUpdate:loading': Function as VuePropType<(val: boolean) => void>,
    onScroll: Function as VuePropType<(value: number) => void>
  },
  slots: Object as SlotsType<{
    default(): void
    backTop(): void
    nav(props: NavBarProps): void
  }>,
  emits: [ 'update:loading', 'scroll', 'navClickLeft' ],
  setup(props, { attrs, slots, expose, emit }) {
    const route = useRoute() as unknown as AppRouteModule
    const router = useRouter()
    const { loading: loadingRef, loadingType, loadingMsg, hiddenSlot } = toRefs(props)

    const backTopEl = ref()
    const viewScrollRoot = ref()
    const initStatus = ref(true)
    const showBackTop = ref(false)
    const hiddenSlotRef = ref(props.hiddenSlot)

    const navTitle = computed(() => route.meta?.name ?? route.meta?.title as string)

    const { y } = useScroll(viewScrollRoot, {
      behavior: 'smooth'
    })

    const [ loading, setLoading ] = usePageLoading({
      defaultType: loadingType.value,
      defaultMessage: loadingMsg.value,
      defaultLoading: loadingRef.value
    })

    watch(() => loading.value, (val) => {
      if (initStatus.value && val) initStatus.value = false
      emit('update:loading', val)
    }, {
      deep: true,
      immediate: true
    })

    watch(() => loadingRef.value, (val) => {
      if (loadingType.value === 'toast') {
        setLoading({
          value: val,
          type: loadingType.value,
          message: loadingMsg.value
        })
      } else if (initStatus.value || !val) {
        setLoading({
          value: val,
          type: loadingType.value,
          message: loadingMsg.value
        })
      }
    })

    watch(() => hiddenSlot.value, (val) => {
      hiddenSlotRef.value = val
    }, {
      immediate: true
    })

    watch(() => y.value, (val) => {
      showBackTop.value = val > props.backTopProps?.count
      props.onScroll?.(val)
    })

    onMounted(() => {
      viewScrollRoot.value = document.querySelector('#basic-layout-content')
      backTopEl.value = document.querySelector('#basic-layout-back-top')
    })

    const changeLoading = ({
      value,
      message,
      hiddenSlot
    }: ChangeLoadingParams<boolean>) => {
      if (loadingType.value === 'toast') {
        hiddenSlotRef.value = !!hiddenSlot
        setLoading({ value, message, type: loadingType.value })
      }
    }

    function navClickLeft() {
      if (props.onNavClickLeft) {
        props.onNavClickLeft()
      } else {
        router.back()
      }
    }

    expose({
      toggleLoading: changeLoading,
      setScrollTop: (val: number) => y.value = val
    })

    return () => {
      const navProps = {
        fixed: true,
        placeholder: true,
        leftArrow: true,
        title: navTitle.value,
        ...(props.navProps || {})
      } as NavBarProps
      const navSlot = getSlot(slots, props, 'nav')
      const backTopRender = getSlotVNode(slots, props, 'backTop')
      return (
        <div class={styles.pageContainer} style={{ backgroundColor: props.bgColor, ...(attrs?.style as any || {}) }}>
          {props.nav !== false && (
            <>
              {isFunction(navSlot)
                ? navSlot?.({ ...navProps })
                : (
                  <NavBar{...navProps} onClickLeft={() => navClickLeft()} />
                )}
            </>
          )}
          <div class={classNames(styles.pageContainerWrapper, props.wrapperClass || '')}>
            {
              loadingType.value === 'skeleton'
                ? (
                  <ProSkeleton line={3} loading={loading.value}>
                    {slots.default?.()}
                  </ProSkeleton>
                )
                : ((!loading.value || !hiddenSlotRef.value) && <>{slots.default?.()}</>)
            }
          </div>
          {
            !!backTopEl.value && showBackTop.value && props.backTop !== false && (
              <Teleport to="#basic-layout-back-top">
                {backTopRender || (
                  <div class={styles.pageBackTop} onClick={() => y.value = 0} style={props.backTopProps?.style}>
                    <Icon name="back-top" class="relative top-1.5px" />
                  </div>
                )}
              </Teleport>
            )
          }
        </div>
      )
    }
  }
})

export default PageContainer
